@@ -40,44 +40,43 @@
 
 module OpenAI.V1
     ( -- * Methods
-      getClientEnv
+      Methods(..)
+    , getClientEnv
     , makeMethods
-    , Methods(..)
-
       -- * Servant
     , API
     ) where
 
+import Control.Monad (foldM)
 import Data.ByteString.Char8 ()
 import Data.Proxy (Proxy(..))
 import OpenAI.Prelude
 import OpenAI.V1.Audio.Speech (CreateSpeech)
-import OpenAI.V1.Embeddings (CreateEmbeddings, EmbeddingObject)
+import OpenAI.V1.Audio.Transcriptions (CreateTranscription, TranscriptionObject)
+import OpenAI.V1.Audio.Translations (CreateTranslation, TranslationObject)
 import OpenAI.V1.Batches (BatchID, BatchObject, CreateBatch)
+import OpenAI.V1.Chat.Completions (ChatCompletionObject, CreateChatCompletion)
 import OpenAI.V1.DeletionStatus (DeletionStatus)
+import OpenAI.V1.Embeddings (CreateEmbeddings, EmbeddingObject)
 import OpenAI.V1.Files (FileID, FileObject, UploadFile)
-import OpenAI.V1.Images.Image (ImageObject)
-import OpenAI.V1.Images.Generations (CreateImage)
 import OpenAI.V1.Images.Edits (CreateImageEdit)
+import OpenAI.V1.Images.Generations (CreateImage)
+import OpenAI.V1.Images.Image (ImageObject)
 import OpenAI.V1.Images.Variations (CreateImageVariation)
 import OpenAI.V1.ListOf (ListOf(..))
 import OpenAI.V1.Message (Message)
 import OpenAI.V1.Models (Model, ModelObject)
 import OpenAI.V1.Moderations (CreateModeration, Moderation)
 import OpenAI.V1.Order (Order)
-import OpenAI.V1.Threads (Thread, ThreadID, ModifyThread, ThreadObject)
+import OpenAI.V1.Responses (CreateResponse, InputItem, ResponseObject)
+import OpenAI.V1.Threads (ModifyThread, Thread, ThreadID, ThreadObject)
+import OpenAI.V1.Threads.Messages (MessageID, MessageObject, ModifyMessage)
 import OpenAI.V1.Threads.Runs.Steps (RunStepObject(..), StepID)
 import Servant.Client (ClientEnv)
 import Servant.Multipart.Client ()
 
 import OpenAI.V1.Assistants
     (AssistantID, AssistantObject, CreateAssistant, ModifyAssistant)
-import OpenAI.V1.Audio.Transcriptions
-    (CreateTranscription, TranscriptionObject)
-import OpenAI.V1.Audio.Translations
-    (CreateTranslation, TranslationObject)
-import OpenAI.V1.Chat.Completions
-    (ChatCompletionObject, CreateChatCompletion)
 import OpenAI.V1.FineTuning.Jobs
     ( CheckpointObject
     , CreateFineTuningJob
@@ -85,8 +84,6 @@ import OpenAI.V1.FineTuning.Jobs
     , FineTuningJobID
     , JobObject
     )
-import OpenAI.V1.Threads.Messages
-    (MessageID, MessageObject, ModifyMessage)
 import OpenAI.V1.Threads.Runs
     ( CreateRun
     , CreateThreadAndRun
@@ -109,39 +106,42 @@ import OpenAI.V1.VectorStores
     , VectorStoreID
     , VectorStoreObject(..)
     )
-import OpenAI.V1.VectorStores.Files
-    ( CreateVectorStoreFile(..)
-    , VectorStoreFileID
-    , VectorStoreFileObject(..)
-    )
 import OpenAI.V1.VectorStores.FileBatches
     ( CreateVectorStoreFileBatch(..)
-    , VectorStoreFilesBatchObject(..)
     , VectorStoreFileBatchID
+    , VectorStoreFilesBatchObject(..)
     )
+import OpenAI.V1.VectorStores.Files
+    (CreateVectorStoreFile(..), VectorStoreFileID, VectorStoreFileObject(..))
 
 import qualified Control.Exception as Exception
+import qualified Data.Aeson as Aeson
+import qualified Data.ByteString as SBS
+import qualified Data.ByteString.Char8 as S8
+import qualified Data.IORef as IORef
 import qualified Data.Text as Text
 import qualified Network.HTTP.Client as HTTP.Client
 import qualified Network.HTTP.Client.TLS as TLS
+import qualified Network.HTTP.Types.Status as Status
 import qualified OpenAI.V1.Assistants as Assistants
 import qualified OpenAI.V1.Audio as Audio
 import qualified OpenAI.V1.Batches as Batches
 import qualified OpenAI.V1.Chat.Completions as Chat.Completions
 import qualified OpenAI.V1.Embeddings as Embeddings
-import qualified OpenAI.V1.FineTuning.Jobs as FineTuning.Jobs
 import qualified OpenAI.V1.Files as Files
+import qualified OpenAI.V1.FineTuning.Jobs as FineTuning.Jobs
 import qualified OpenAI.V1.Images as Images
 import qualified OpenAI.V1.Models as Models
 import qualified OpenAI.V1.Moderations as Moderations
-import qualified OpenAI.V1.Threads.Runs as Threads.Runs
-import qualified OpenAI.V1.Threads.Runs.Steps as Threads.Runs.Steps
+import qualified OpenAI.V1.Responses as Responses
 import qualified OpenAI.V1.Threads as Threads
 import qualified OpenAI.V1.Threads.Messages as Messages
+import qualified OpenAI.V1.Threads.Runs as Threads.Runs
+import qualified OpenAI.V1.Threads.Runs.Steps as Threads.Runs.Steps
 import qualified OpenAI.V1.Uploads as Uploads
 import qualified OpenAI.V1.VectorStores as VectorStores
-import qualified OpenAI.V1.VectorStores.Files as VectorStores.Files
 import qualified OpenAI.V1.VectorStores.FileBatches as VectorStores.FileBatches
+import qualified OpenAI.V1.VectorStores.Files as VectorStores.Files
 import qualified OpenAI.V1.VectorStores.Status as VectorStores.Status
 import qualified Servant.Client as Client
 
@@ -168,8 +168,12 @@ makeMethods
     -- ^
     -> Text
     -- ^ API token
+    -> Maybe Text
+    -- ^ Organization ID
+    -> Maybe Text
+    -- ^ Project ID
     -> Methods
-makeMethods clientEnv token = Methods{..}
+makeMethods clientEnv token organizationID projectID = Methods{..}
   where
     authorization = "Bearer " <> token
 
@@ -178,6 +182,11 @@ makeMethods clientEnv token = Methods{..}
             :<|>  createTranslation_
                     )
       :<|>  createChatCompletion
+      :<|>  (     createResponse
+            :<|>  retrieveResponse
+            :<|>  cancelResponse
+            :<|>  listResponseInputItems_
+            )
       :<|>  createEmbeddings_
       :<|>  (     createFineTuningJob
             :<|>  listFineTuningJobs_
@@ -250,6 +259,7 @@ makeMethods clientEnv token = Methods{..}
                 :<|>  retrieveRunStep
                 )
             )
+
       :<|>  (   (\x -> x "assistants=v2")
             ->  (     createVectorStore
                 :<|>  listVectorStores_
@@ -272,7 +282,7 @@ makeMethods clientEnv token = Methods{..}
                 :<|>  listVectorStoreFilesInABatch_
                 )
             )
-      ) = Client.hoistClient @API Proxy run (Client.client @API Proxy) authorization
+      ) = Client.hoistClient @API Proxy run (Client.client @API Proxy) authorization organizationID projectID
 
     run :: Client.ClientM a -> IO a
     run clientM = do
@@ -310,6 +320,137 @@ makeMethods clientEnv token = Methods{..}
         toVector (listVectorStoreFiles_ a b c d e f)
     listVectorStoreFilesInABatch a b c d e f g =
         toVector (listVectorStoreFilesInABatch_ a b c d e f g)
+    listResponseInputItems a = toVector (listResponseInputItems_ a)
+
+    -- Streaming implementation using http-client and SSE parsing
+    createResponseStream req onEvent = do
+        let req' = req{ Responses.stream = Just True }
+        ssePostJSON "/v1/responses" req' onEvent
+
+    createResponseStreamTyped
+        :: CreateResponse
+        -> (Either Text Responses.ResponseStreamEvent -> IO ())
+        -> IO ()
+    createResponseStreamTyped req onEvent =
+        createResponseStream req $ \ev -> case ev of
+            Left err -> onEvent (Left err)
+            Right val -> case Aeson.fromJSON val of
+                Aeson.Error msg -> onEvent (Left (Text.pack msg))
+                Aeson.Success e -> onEvent (Right e)
+
+    ssePostJSON :: ToJSON a
+                => String
+                -> a
+                -> (Either Text Aeson.Value -> IO ())
+                -> IO ()
+    ssePostJSON path body onEvent = do
+        let base = Client.baseUrl clientEnv
+        let secure = case Client.baseUrlScheme base of
+                Client.Http -> False
+                Client.Https -> True
+        let host = S8.pack (Client.baseUrlHost base)
+        let port = Client.baseUrlPort base
+        let basePath = Client.baseUrlPath base
+        let fullPath = S8.pack (normalizePath basePath <> path)
+
+        let headers0 =
+                [ ("Authorization", S8.pack (Text.unpack authorization))
+                , ("Accept", "text/event-stream")
+                , ("Content-Type", "application/json")
+                ]
+        let headers1 = case organizationID of
+                Nothing -> headers0
+                Just org -> ("OpenAI-Organization", S8.pack (Text.unpack org)) : headers0
+        let headers = case projectID of
+                Nothing -> headers1
+                Just proj -> ("OpenAI-Project", S8.pack (Text.unpack proj)) : headers1
+
+        let request = HTTP.Client.defaultRequest
+                { HTTP.Client.secure = secure
+                , HTTP.Client.host = host
+                , HTTP.Client.port = port
+                , HTTP.Client.method = "POST"
+                , HTTP.Client.path = fullPath
+                , HTTP.Client.requestHeaders = headers
+                , HTTP.Client.requestBody = HTTP.Client.RequestBodyLBS (Aeson.encode body)
+                , HTTP.Client.responseTimeout = HTTP.Client.responseTimeoutNone
+                }
+
+        HTTP.Client.withResponse request (Client.manager clientEnv) $ \response -> do
+            -- Short-circuit on non-2xx HTTP statuses and surface a single error event
+            let st = HTTP.Client.responseStatus response
+            if not (Status.statusIsSuccessful st)
+                then do
+                    bodyChunks <- HTTP.Client.brConsume (HTTP.Client.responseBody response)
+                    let errBody = SBS.concat bodyChunks
+                    let msg =
+                            "HTTP error "
+                            <> renderIntegral (Status.statusCode st)
+                            <> " "
+                            <> (Text.pack (S8.unpack (Status.statusMessage st)))
+                            <> (if SBS.null errBody then "" else ": " <> Text.pack (S8.unpack errBody))
+                    onEvent (Left msg)
+                else do
+                    let br = HTTP.Client.responseBody response
+                    lineBufRef <- IORef.newIORef SBS.empty
+                    eventBufRef <- IORef.newIORef ([] :: [SBS.ByteString])
+                    let flushEvent = do
+                            es <- IORef.atomicModifyIORef eventBufRef (\buf -> ([], reverse buf))
+                            if null es
+                                then pure False
+                                else do
+                                    let payload = S8.concat es
+                                    if payload == "[DONE]"
+                                        then pure True
+                                        else case (Aeson.eitherDecodeStrict payload :: Either String Aeson.Value) of
+                                            Left err -> onEvent (Left (Text.pack err)) >> pure False
+                                            Right val -> onEvent (Right val) >> pure False
+
+                    -- Note: SSE frames can include fields like "event:" and others.
+                    -- We currently ignore all non-"data:" fields and only buffer
+                    -- "data:" lines; an empty line flushes a complete event.
+                    let handleLine line = do
+                            let l = stripCR line
+                            if S8.null l
+                                then flushEvent
+                                else if "data:" `S8.isPrefixOf` l
+                                    then do
+                                        let d = S8.dropWhile (==' ') (S8.drop 5 l)
+                                        IORef.modifyIORef' eventBufRef (d:)
+                                        pure False
+                                    else pure False
+
+                    let loop = do
+                            chunk <- HTTP.Client.brRead br
+                            if SBS.null chunk
+                                then do
+                                    -- flush any pending event at EOF
+                                    _ <- flushEvent
+                                    pure ()
+                                else do
+                                    prev <- IORef.readIORef lineBufRef
+                                    let combined = prev <> chunk
+                                    let ls = S8.split '\n' combined
+                                    case unsnoc ls of
+                                        Nothing -> loop
+                                        Just (completeLines, lastLine) -> do
+                                            IORef.writeIORef lineBufRef lastLine
+                                            stop <- foldM (\acc ln -> if acc then pure True else handleLine ln) False completeLines
+                                            if stop then pure () else loop
+
+                    loop
+
+    normalizePath p = case p of
+        "" -> ""
+        ('/':_) -> p
+        _ -> '/':p
+
+    stripCR bs = case S8.unsnoc bs of
+        Just (initBs, '\r') -> initBs
+        _ -> bs
+
+    unsnoc [] = Nothing
+    unsnoc xs = Just (init xs, last xs)
 
 -- | Hard-coded boundary to simplify the user-experience
 --
@@ -324,7 +465,16 @@ data Methods = Methods
     , createTranscription :: CreateTranscription -> IO TranscriptionObject
     , createTranslation :: CreateTranslation -> IO TranslationObject
     , createChatCompletion :: CreateChatCompletion -> IO ChatCompletionObject
+    , createResponse :: CreateResponse -> IO ResponseObject
+    , createResponseStream
+        :: CreateResponse
+        -> (Either Text Aeson.Value -> IO ())
+        -> IO ()
     , createEmbeddings :: CreateEmbeddings -> IO (Vector EmbeddingObject)
+    , createResponseStreamTyped
+        :: CreateResponse
+        -> (Either Text Responses.ResponseStreamEvent -> IO ())
+        -> IO ()
     , createFineTuningJob :: CreateFineTuningJob -> IO JobObject
     , listFineTuningJobs
         :: Maybe Text
@@ -462,6 +612,9 @@ data Methods = Methods
         -> Maybe Text
         -- ^ include[]
         -> IO RunStepObject
+    , retrieveResponse :: Text -> IO ResponseObject
+    , cancelResponse :: Text -> IO ResponseObject
+    , listResponseInputItems :: Text -> IO (Vector InputItem)
     , createVectorStore :: CreateVectorStore -> IO VectorStoreObject
     , listVectorStores
         :: Maybe Natural
@@ -530,9 +683,12 @@ data Methods = Methods
 -- | Servant API
 type API
     =   Header' [ Required, Strict ] "Authorization" Text
+    :>  Header' [ Optional, Strict ] "OpenAI-Organization" Text
+    :>  Header' [ Optional, Strict ] "OpenAI-Project" Text
     :>  "v1"
     :>  (     Audio.API
         :<|>  Chat.Completions.API
+        :<|>  Responses.API
         :<|>  Embeddings.API
         :<|>  FineTuning.Jobs.API
         :<|>  Batches.API
